#include "app_gtk.h"

extern GtkWidget *app;

void close_app(GtkWidget  *window, gpointer data)
{
	gtk_main_quit();
} 

void about_app(GtkMenuItem *menuItem, gpointer user_data)
{
	gtk_show_about_dialog(GTK_WINDOW(app), "version", "Verze 1.0.0",
										   "comments", "Aplikace na sledování základních informací o systému a vytížení systémových zdrojů",
										   "program-name", "System status check", 
											NULL);
}

void *
start_printing (GtkWidget *do_widget, gpointer user_data)
{

  GtkPrintOperation *operation;
  PrintData *data;	
  GError *error = NULL;

  gint res;

  operation = gtk_print_operation_new ();
  data = g_new0 (PrintData, 1);

  data->heading = "Zpráva o stavu systému";
  data->font_size = 12.0;

  g_signal_connect (G_OBJECT (operation), "begin-print",
		    G_CALLBACK (begin_print), data);
  g_signal_connect (G_OBJECT (operation), "draw-page",
		    G_CALLBACK (draw_page), data);
  g_signal_connect (G_OBJECT (operation), "end-print",
		    G_CALLBACK (end_print), data);	

  res = gtk_print_operation_run(operation, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW (app), &error);

	if(res == GTK_PRINT_OPERATION_RESULT_ERROR)
	{
		GtkWidget *err_dialog;
		err_dialog = gtk_message_dialog_new(GTK_WINDOW(app), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Printing failed:%s\n ", error->message);

		g_signal_connect(err_dialog, "response", 
				G_CALLBACK(gtk_widget_destroy), NULL);

		gtk_widget_show(err_dialog);
		g_error_free(error);
	}

  g_object_unref (operation);
	
  return NULL;
}

void
begin_print (GtkPrintOperation *operation,
	     GtkPrintContext   *context,
	     gpointer           user_data)
{
  char *contents;
  int i;
  double height;
  PrintData *data = (PrintData *)user_data;
float avCpu = 0.0, avRam = 0.0, medianCpu, medianRam;

  for(i = 0; i < swhw_ptr->cpu.history_index; i++)
	 avCpu+=swhw_ptr->cpu.history_utilization[i];

	avCpu/=swhw_ptr->cpu.history_index;
	medianCpu = get_median(swhw_ptr->cpu.history_utilization, swhw_ptr->cpu.history_index);

   for(i = 0; i < swhw_ptr->ram.history_index; i++)
	 avRam+=swhw_ptr->ram.history_usage[i];

	avRam/=(swhw_ptr->ram.history_index-1);
	medianRam = get_median(swhw_ptr->ram.history_usage, swhw_ptr->ram.history_index-1);

  contents = g_strdup_printf("Systém:\n  OS:\t\t %s\n  Verze jádra:\t %s\n  Distribuce:\t %s\n  Verze\t\t %s\n\nProcesor:\n  Model:\t%s\n\
  Frekvence:\t %.0f MHz\n  Počet jader:\t %d\n  Cache:\t%s\n  Procesy:\t %d\n  Vytížení CPU:\t průměr %.1f%%, medián %.1f%%\n\nRAM:\n\
  Celková:\t %.0f MB\n  Dostupná:\t %.0f MB\n  Swap:\t\t %.0f/%.0f MB\n  Využití RAM:\t průměr %.1f%%, medián %.1f%%\n\nOstatní:\n  Graf. karta:\t %s\n  Zákl. deska:\t %s\n\n",
		 swhw_ptr->os, swhw_ptr->kernelRelease, swhw_ptr->distro, swhw_ptr->v_distro,swhw_ptr->cpu.nazev,
		 swhw_ptr->cpu.frekvence, swhw_ptr->cpu.num_of_cores, swhw_ptr->cpu.cache_size, swhw_ptr->cnt_proc, avCpu, medianCpu,
		 swhw_ptr->ram.totalRam, swhw_ptr->ram.avRam, swhw_ptr->ram.swapTotal-swhw_ptr->ram.swapFree, swhw_ptr->ram.swapTotal, avRam, medianRam,
		 swhw_ptr->g_adapter, swhw_ptr->board);
	
   height = gtk_print_context_get_height (context) - HEADER_HEIGHT - 10;
   data->lines_per_page = floor (height / data->font_size);

   data->lines = g_strsplit (contents, "\n", 0);
   g_free (contents);

  i = 0;
  while (data->lines[i] != NULL)
    i++;

  data->num_lines = i;
  data->num_pages = (data->num_lines - 1) / data->lines_per_page + 1;

  gtk_print_operation_set_n_pages (operation, data->num_pages);
}

void
draw_page (GtkPrintOperation *operation,
	   GtkPrintContext   *context,
	   gint               page_nr,
	   gpointer           user_data)
{ 
  cairo_t *cr;
  PangoLayout *layout;
  gint text_width, text_height;
  gdouble width, height;
  gint line, i;
  PangoFontDescription *desc;
  gchar *page_str;
  PrintData *data = (PrintData *)user_data;

  cr = gtk_print_context_get_cairo_context (context);
  width = gtk_print_context_get_width (context);
  height = gtk_print_context_get_height (context);

  cairo_rectangle (cr, 0, 0, width, height);
  cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
  cairo_set_source_rgb (cr, 0, 0, 0);
  cairo_set_line_width (cr, 1);
  cairo_stroke (cr);

  layout = gtk_print_context_create_pango_layout (context);

  desc = pango_font_description_from_string ("sans 14");
  pango_layout_set_font_description (layout, desc);
  pango_font_description_free (desc);

  pango_layout_set_text (layout, data->heading, -1);									
  pango_layout_get_pixel_size (layout, &text_width, &text_height);

  cairo_move_to (cr, (width - text_width) / 2,  (HEADER_HEIGHT - text_height) );	
  pango_cairo_show_layout (cr, layout);


  page_str = g_strdup_printf ("%d/%d", page_nr + 1, data->num_pages);				
  pango_layout_set_text (layout, page_str, -1);
  g_free (page_str);

  pango_layout_set_width (layout, -1);
  pango_layout_get_pixel_size (layout, &text_width, &text_height);
  cairo_move_to (cr, width / 2, height);											
  pango_cairo_show_layout (cr, layout);

  g_object_unref (layout);

  layout = gtk_print_context_create_pango_layout (context);

  desc = pango_font_description_from_string ("monospace");
  pango_font_description_set_size (desc, data->font_size * PANGO_SCALE);
  pango_layout_set_font_description (layout, desc);
  pango_font_description_free (desc);


  cairo_move_to (cr, 15, HEADER_HEIGHT + 20);
  line = page_nr * data->lines_per_page;

  for (i = 0; i < data->lines_per_page && line < data->num_lines; i++)
  {
      pango_layout_set_text (layout, data->lines[line], -1);
      pango_cairo_show_layout (cr, layout);
      cairo_rel_move_to (cr, 0, data->font_size + 5);
	  line++;
  }
	  
  g_object_unref (layout);
}

void
end_print (GtkPrintOperation *operation,
	   GtkPrintContext   *context,
	   gpointer           user_data)
{

  PrintData *data = (PrintData *)user_data;

  g_strfreev (data->lines);
  g_free (data);
}

gboolean draw_callback_cpu(GtkWidget *widget, cairo_t *cr, gpointer user_data)
{
	gdouble dif;
	int j;

	dif = graphGrid(widget, cr);
	cairo_set_line_width (cr, 2.0);
	cairo_set_source_rgba (cr, 1, 0.2, 0.2, 0.6);

	for (j = 0; j < swhw_ptr->cpu.history_index; j++ )  
       	cairo_line_to(cr, -j*(10.0 + dif), -(swhw_ptr->cpu.history_utilization[j]/100.0)*GHEIGHT); 
    
	cairo_stroke (cr);

  return FALSE;
}

gboolean repaint_cpu(gpointer user_data)
{
    gchar *text;
	int k;
    cpu_utilization();    
	GtkWidget **timeLabel = user_data;
	
	for(k = 0; k < swhw_ptr->cpu.num_of_cores + 2; k++)
	{
		 if(k == 0){
			text = g_strdup_printf("CPU utilization: %.1f%%", swhw_ptr->cpu.core[k].utilization);
			gtk_label_set_text(GTK_LABEL(timeLabel[k]), text);
		 }
		 else if (k < swhw_ptr->cpu.num_of_cores + 1)
		 {
			text = g_markup_printf_escaped("<span font_desc=\"11.0\"><b>CPU%d:</b>%5.1f%%</span>", k, swhw_ptr->cpu.core[k].utilization);
			gtk_label_set_markup(GTK_LABEL(timeLabel[k]), text);
		 }
		 else{
			text = g_markup_printf_escaped("<span font=\"Courier new 11.0\"><b>Processes</b> %*c:  %d</span>", 5 ,' ',swhw_ptr->cnt_proc);
			gtk_label_set_markup(GTK_LABEL(timeLabel[k]), text);
		 }
		 g_free(text);
	}

	for(k = swhw_ptr->cpu.history_index - 1; k > 0; k--)
		swhw_ptr->cpu.history_utilization[k]=swhw_ptr->cpu.history_utilization[k-1];

    swhw_ptr->cpu.history_utilization[0] = swhw_ptr->cpu.core[0].utilization;

    if(swhw_ptr->cpu.history_index <= 60)
        swhw_ptr->cpu.history_index++;
 
    return TRUE;
}

gdouble graphGrid(GtkWidget *widget, cairo_t *cr){
        
	GdkRectangle da;     
    GdkWindow *window = gtk_widget_get_window(widget);

    gdk_window_get_geometry (window, &da.x, &da.y, &da.width, &da.height);
	cairo_translate (cr, da.width, da.height);    

    gdouble x1 = 0.0, y1 = 0.0, x2 = 0.0, y2 = 0.0, setGridX = 0.0, setGridY = 0.0, space1, space2;

    cairo_clip_extents (cr, &x1, &y1, &x2, &y2);

    cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);	
    cairo_paint (cr);

    cairo_set_line_width (cr, 0.3);
    cairo_set_source_rgba (cr, 5, 10, 2, 0.6);
    space1 = -y1/5;
    space2 = -x1/6;
	int j;
    for (j = 0; j < 6; j++)
    {
        if(j < 5)
        {
            cairo_move_to (cr, x1, -setGridX );
            cairo_line_to (cr, x2, -setGridX );
            setGridX += space1;
        }
        cairo_move_to (cr, -setGridY, y1);
        cairo_line_to (cr, -setGridY, y2);
        setGridY += space2;            
    }
    cairo_stroke (cr);
	
	return (da.width-GWIDTH)/60.0;
}

gboolean draw_callback_ram(GtkWidget *widget, cairo_t *cr, gpointer user_data)
{
	gdouble dif;
	dif = graphGrid(widget, cr);

    cairo_set_line_width (cr, 2.0);
    cairo_set_source_rgba (cr, 0.2, 1, 0.2, 0.6);

	int j;
	for (j = 0; j < swhw_ptr->ram.history_index; j++ )   
       cairo_line_to(cr, j*(-10.0-dif), -(swhw_ptr->ram.history_usage[j]/100.0)*GHEIGHT);

    cairo_stroke(cr);  

	return FALSE;  
}

gboolean repaint_ram(gpointer user_data)
{
    gchar *text;
	float ram_u;
    int k;
	ram_u = ram_usage();
	GtkWidget **ram_label = user_data;

	for(k = 0; k < 3; k++)
	{
		 if(k == 0){
			text = g_strdup_printf("RAM usage: %.1f%%", ram_u);
			gtk_label_set_text(GTK_LABEL(ram_label[k]), text);
		 }
		 else if(k == 1)
		 {
			text = g_markup_printf_escaped("<span font_desc=\"11.0\"><b>Avaliable RAM: </b>%5.0f MB <b>/</b> %5.0f MB</span>",
														 swhw_ptr->ram.avRam, swhw_ptr->ram.totalRam);
			gtk_label_set_markup(GTK_LABEL(ram_label[k]), text);
		 }
		else	
		{
			text = g_markup_printf_escaped("<span font_desc=\"11.0\"><b>Used Swap: </b>%5.0f MB <b>/</b> %5.0f MB</span>",
														 swhw_ptr->ram.swapTotal - swhw_ptr->ram.swapFree, swhw_ptr->ram.swapTotal);
			gtk_label_set_markup(GTK_LABEL(ram_label[k]), text);
		 }	
		 g_free(text);
	}

	for(k = swhw_ptr->ram.history_index-1; k > 0; k--)
        swhw_ptr->ram.history_usage[k]=swhw_ptr->ram.history_usage[k-1];

    swhw_ptr->ram.history_usage[0] = ram_u;

    if(swhw_ptr->ram.history_index <= 60)
        swhw_ptr->ram.history_index++;
 
    return TRUE;
}

gboolean refresh_proc(gpointer user_data)
{
	GtkTreeIter iter;
	GtkListStore *store = (GtkListStore *)user_data;

  	get_process();
	int i = 0, k;
	int new = 0;
	gboolean valid;
	gint pid = 0;
	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
	while(valid)
	{
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, COLUMN_PID, &pid, -1);
		if(new < swhw_ptr->cnt_proc){
		if(pid == swhw_ptr->proc[new].pid )
		{
			gtk_list_store_set(store, &iter,
						  COLUMN_PID, swhw_ptr->proc[new].pid,
                          COLUMN_NAME, swhw_ptr->proc[new].name,
                          COLUMN_STATUS, swhw_ptr->proc[new].status, 
						  COLUMN_MEMORY, swhw_ptr->proc[new].memory,                      
                         -1);
			new++;
			valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);	
		}
		else if(pid < swhw_ptr->proc[new].pid){
			valid = gtk_list_store_remove(store, &iter);
		}
		}
		else {
			valid = gtk_list_store_remove(store, &iter);		
		}

		i++;	
	 }
	 for (k = new; k < swhw_ptr->cnt_proc; k++)
     {
		 gtk_list_store_append (store, &iter);
      	 gtk_list_store_set (store, &iter,
                          COLUMN_PID, swhw_ptr->proc[k].pid,
                          COLUMN_NAME, swhw_ptr->proc[k].name,
                          COLUMN_STATUS, swhw_ptr->proc[k].status,   
						  COLUMN_MEMORY, swhw_ptr->proc[k].memory,                    
                         -1);
	 }
    return TRUE;
}

void quit_proces (GtkWidget *button, gpointer data)
{
	 GtkTreeIter iter;
  	 GtkTreeView *treeview = (GtkTreeView *)data;
   	 GtkTreeModel *model = gtk_tree_view_get_model (treeview);
  	 GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
	 int pid;

  	 if (gtk_tree_selection_get_selected (selection, NULL, &iter))
  	 {
	  	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, COLUMN_PID, &pid, -1);
	  	if(kill(pid, SIGKILL)){
      		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
	  		get_process();
		}
  	}
}

gboolean refresh_disk (GtkWidget *button, gpointer data)
{
  	GtkTreeIter iter;
	GtkTreeView *treeview = (GtkTreeView *)data;
	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
	
	gtk_list_store_clear(GTK_LIST_STORE(model));
	get_disk();
	int i = 0;
	for (i = 0; i < swhw_ptr->cnt_disk; i++)
    {
      	gtk_list_store_append (GTK_LIST_STORE(model), &iter);
      	gtk_list_store_set (GTK_LIST_STORE(model), &iter,
                          COLUMN_SYSTEM, swhw_ptr->disk[i].disk_name,
						  COLUMN_TYPE, swhw_ptr->disk[i].type,
                          COLUMN_CAPACITY, swhw_ptr->disk[i].capacity,
                          COLUMN_USED, swhw_ptr->disk[i].used_space,
						  COLUMN_FREE,  swhw_ptr->disk[i].free_space,
    					  COLUMN_DESTINATION, swhw_ptr->disk[i].connect_to,                     
                          -1);
    }
	return TRUE;	
}
