#include "app_gtk.h"

GtkWidget *app = NULL;
GtkTreeModel *model = NULL;

GtkWidget *addLabelRow(gchar *leftL, gchar *rightL, int spacing)
{
	GtkWidget *leftLabel;
	char *markup;
	int lengthstr = strlen(leftL);

	leftLabel = gtk_label_new(NULL);
	markup = g_markup_printf_escaped("<span font=\"Courier new 11.0\"><b>%s</b> %*c  : %s</span>", leftL, spacing-lengthstr,' ',rightL);
	gtk_label_set_markup(GTK_LABEL(leftLabel), markup);
	g_free(markup);

	gtk_label_set_xalign(GTK_LABEL(leftLabel), 0.0); 

	return leftLabel;
}


GtkWidget *createGraph(gchar *xy_start, gchar *y_end, gchar *x_end, GtkWidget *title, GtkWidget *area)
{
	GtkWidget *axisX_end, *axisX_start, *axisY_end;	
	GtkWidget *vbox1, *vbox2, *vbox4, *vbox5;	
		
	axisX_end=gtk_label_new(x_end);
    axisX_start=gtk_label_new(xy_start); 
    axisY_end=gtk_label_new(y_end);
	
    vbox1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
    vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
    vbox4 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,0);
    vbox5 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,0);

	gtk_box_pack_start(GTK_BOX(vbox2), vbox5, FALSE, FALSE, 0);   
  	gtk_box_pack_start(GTK_BOX(vbox2), area, FALSE, FALSE, 0);  
   	gtk_box_pack_start(GTK_BOX(vbox2), vbox4, FALSE, FALSE, 0);  

	gtk_box_pack_start(GTK_BOX(vbox4), axisX_end, FALSE, FALSE, 0); 
    gtk_box_pack_start(GTK_BOX(vbox5), title, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox5), axisY_end, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox4), axisX_start, TRUE, TRUE, 0);

	gtk_label_set_xalign(GTK_LABEL(axisX_start), 1.0);
    gtk_label_set_xalign(GTK_LABEL(axisY_end), 1.0);
   	gtk_box_pack_start(GTK_BOX(vbox1), vbox2, TRUE, TRUE, 10);
   	
	return vbox1;
}

static void add_columns_filesystem (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("File system", renderer, "text", COLUMN_SYSTEM, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Type", renderer, "text", COLUMN_TYPE, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Capacity", renderer, "text", COLUMN_CAPACITY, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Used space", renderer, "text", COLUMN_USED, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Free space", renderer, "text", COLUMN_FREE, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Connected to", renderer, "text", COLUMN_DESTINATION, NULL);
  gtk_tree_view_append_column (treeview, column);

}

static GtkTreeModel *new_model_filesystem (void)
{
  gint i = 0;
  GtkListStore *store;
  GtkTreeIter iter;

  store = gtk_list_store_new (NUMBER_COLUMNS,
                              G_TYPE_STRING,
							  G_TYPE_STRING,
                              G_TYPE_STRING,
                              G_TYPE_STRING,
							  G_TYPE_STRING,
							  G_TYPE_STRING);
	
	for (i = 0; i < swhw_ptr->cnt_disk; i++)
    {
      gtk_list_store_append (store, &iter);
      gtk_list_store_set (store, &iter,
                          COLUMN_SYSTEM, swhw_ptr->disk[i].disk_name,
						  COLUMN_TYPE, swhw_ptr->disk[i].type,
                          COLUMN_CAPACITY, swhw_ptr->disk[i].capacity,
                          COLUMN_USED, swhw_ptr->disk[i].used_space,
						  COLUMN_FREE,  swhw_ptr->disk[i].free_space,
    					  COLUMN_DESTINATION, swhw_ptr->disk[i].connect_to,                     
                          -1);
    }

  return GTK_TREE_MODEL (store);
}

static GtkWidget *createMemoryPage(){
	
	GtkWidget *memory_page, *graph_ram_area, *hboxRam;	
	GtkWidget *treeview, *hbox, *vbox, *hbox_button, *scrollwindow , *labelFileSystem, *refreshButton;
	GtkTreeModel * filesystemModel;

	graph_ram_area = gtk_drawing_area_new ();    
    gtk_widget_set_size_request(graph_ram_area,GWIDTH,GHEIGHT);
	
	memory_page = gtk_box_new (GTK_ORIENTATION_VERTICAL,10);
	hboxRam = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);

	GtkWidget **ram_usage_label;
	ram_usage_label = g_new(GtkWidget *, 3);

	int i;
	for(i = 0; i < 3; i++){
		ram_usage_label[i] = gtk_label_new(NULL);
		if(i > 0)
			gtk_box_pack_start(GTK_BOX(hboxRam), ram_usage_label[i], TRUE, TRUE, 0);
	}

	gtk_box_pack_start(GTK_BOX(memory_page), createGraph("0", "100%", "60 [s]", ram_usage_label[0], graph_ram_area), FALSE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(memory_page), hboxRam, FALSE, FALSE, 0);


	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	hbox_button = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	 
	scrollwindow = gtk_scrolled_window_new (NULL, NULL);						//vytvoreni rolovani
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwindow),
                                           GTK_SHADOW_ETCHED_IN);

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwindow),		//vlastnosti rolovatka
                                      GTK_POLICY_NEVER,
                                      GTK_POLICY_AUTOMATIC);
	 
	labelFileSystem = gtk_label_new("Mounted file systems:");
 	refreshButton = gtk_button_new_from_icon_name("gtk-refresh", GTK_ICON_SIZE_BUTTON);
	gtk_label_set_xalign(GTK_LABEL(labelFileSystem), 0.0);
	gtk_label_set_yalign(GTK_LABEL(labelFileSystem), 1.0);
	 
	filesystemModel = new_model_filesystem ();										//vytvoreni noveho modelu
    treeview = gtk_tree_view_new_with_model (filesystemModel);						//vytvoreni treeview
    g_object_unref (filesystemModel);

 	add_columns_filesystem (GTK_TREE_VIEW (treeview));								//pridani sloupcu do treeview
	
	gtk_box_pack_start(GTK_BOX(hbox_button),  labelFileSystem, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox_button),  refreshButton, FALSE, FALSE, 0);	 
	gtk_box_pack_start(GTK_BOX(vbox),  hbox_button, FALSE, FALSE, 5);
	gtk_container_add (GTK_CONTAINER (scrollwindow), treeview);						//pridani treeview do rolovaciho okna
	gtk_box_pack_start(GTK_BOX(vbox), scrollwindow, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 10);
	gtk_box_pack_start(GTK_BOX(memory_page), hbox, TRUE, TRUE, 10);

	g_signal_connect (G_OBJECT (refreshButton),"clicked", G_CALLBACK (refresh_disk), treeview);
	g_signal_connect (G_OBJECT (graph_ram_area),"draw", G_CALLBACK (draw_callback_ram), NULL);
	
	g_timeout_add(1000, repaint_ram, ram_usage_label);

	return memory_page;
}

static GtkWidget *createCpuPage()
{
	GtkWidget *cpu_page, *graph_cpu_area, *cpu_cores_hbox;	
	graph_cpu_area = gtk_drawing_area_new ();    
    gtk_widget_set_size_request(graph_cpu_area,GWIDTH,GHEIGHT);
	char *labeltext;
	GtkWidget **timeLabel;
	timeLabel = g_new(GtkWidget *, swhw_ptr->cpu.num_of_cores + 2);

	cpu_page = gtk_box_new (GTK_ORIENTATION_VERTICAL,10);	
	cpu_cores_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);	


	int i;
	for(i = 0; i < swhw_ptr->cpu.num_of_cores + 2; i++){

		timeLabel[i] = gtk_label_new(NULL);
		if(i > 0 && i < swhw_ptr->cpu.num_of_cores + 1)
			gtk_box_pack_start(GTK_BOX(cpu_cores_hbox), timeLabel[i], TRUE, TRUE, 0);
	}

	gtk_box_pack_start(GTK_BOX(cpu_page), createGraph("0", "100%", "60 [s]", timeLabel[0], graph_cpu_area), FALSE, FALSE, 10);
	gtk_box_pack_start(GTK_BOX(cpu_page), cpu_cores_hbox, FALSE, FALSE, 0);

	
	GtkWidget *vbox, *hbox;
	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL,0);
	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,0);

	gtk_box_pack_start (GTK_BOX (vbox), addLabelRow("Model name", swhw_ptr->cpu.nazev, 12), FALSE, FALSE, 5);

	labeltext = g_strdup_printf(" %.0f MHz", swhw_ptr->cpu.frekvence);	
	gtk_box_pack_start (GTK_BOX (vbox), addLabelRow("Frequency", labeltext, 12), FALSE, FALSE, 5);
	g_free(labeltext);

	labeltext = g_strdup_printf(" %d", swhw_ptr->cpu.num_of_cores);
	gtk_box_pack_start (GTK_BOX (vbox), addLabelRow("Cores", labeltext, 12), FALSE, FALSE, 5);
	g_free(labeltext);

	gtk_box_pack_start (GTK_BOX (vbox), addLabelRow("L2 cache", swhw_ptr->cpu.cache_size, 12), FALSE, FALSE, 5);
	gtk_box_pack_start (GTK_BOX (vbox), timeLabel[swhw_ptr->cpu.num_of_cores + 1] , FALSE, FALSE, 5);
	gtk_label_set_xalign(GTK_LABEL(timeLabel[swhw_ptr->cpu.num_of_cores + 1]), 0.0);

	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 40);
	gtk_box_pack_start(GTK_BOX(cpu_page), hbox, FALSE, FALSE, 35);

	g_signal_connect (G_OBJECT (graph_cpu_area),"draw", G_CALLBACK (draw_callback_cpu), NULL);	

	g_timeout_add(1000, repaint_cpu, timeLabel);

	return cpu_page;
}

static void add_columns (GtkTreeView *treeview)
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("PID", renderer, "text", COLUMN_PID, NULL);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Name", renderer, "text", COLUMN_NAME, NULL);
 gtk_tree_view_column_set_expand(column, TRUE);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Status", renderer, "text", COLUMN_STATUS, NULL);
 gtk_tree_view_column_set_expand(column, TRUE);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Resident memory", renderer, "text", COLUMN_MEMORY, NULL);
 gtk_tree_view_column_set_expand(column, TRUE);
  gtk_tree_view_append_column (treeview, column);

  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Virtual memory", renderer, "text", COLUMN_MEMORYVM, NULL);
 gtk_tree_view_column_set_expand(column, TRUE);
  gtk_tree_view_append_column (treeview, column);

}

static GtkTreeModel *new_model (void)
{
  gint i = 0;
  GtkListStore *store;
  GtkTreeIter iter;

  store = gtk_list_store_new (NUM_COLUMNS,
                              G_TYPE_UINT,
                              G_TYPE_STRING,
                              G_TYPE_STRING,
							  G_TYPE_STRING,
							  G_TYPE_STRING);
	get_process();

	for (i = 0; i < swhw_ptr->cnt_proc; i++)
    {
      gtk_list_store_append (store, &iter);
      gtk_list_store_set (store, &iter,
                          COLUMN_PID, swhw_ptr->proc[i].pid,
                          COLUMN_NAME, swhw_ptr->proc[i].name,
                          COLUMN_STATUS, swhw_ptr->proc[i].status,
						  COLUMN_MEMORY, swhw_ptr->proc[i].memory,
						  COLUMN_MEMORYVM, swhw_ptr->proc[i].memoryVm,                      
                          -1);
    }

  g_timeout_add(5000, refresh_proc, store);

  return GTK_TREE_MODEL (store);
}

static GtkWidget *createProcessPage()
{
	GtkWidget *process_page;	
    GtkWidget *scrollwindow;
    GtkWidget *treeview;
	GtkWidget *vbox, *buttonvbox;
	GtkWidget *button;
	GtkWidget *searchent;

	process_page = gtk_box_new (GTK_ORIENTATION_HORIZONTAL,0);	
    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	buttonvbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);

	button = gtk_button_new_with_label("Ukončit proces");
	searchent = gtk_search_entry_new();

    scrollwindow = gtk_scrolled_window_new (NULL, NULL);						//vytvoreni rolovani
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollwindow),
                                           GTK_SHADOW_ETCHED_IN);

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwindow),		//vlastnosti rolovatka
                                      GTK_POLICY_NEVER,
                                      GTK_POLICY_AUTOMATIC);
	 
	gtk_box_pack_start (GTK_BOX (buttonvbox), button, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), searchent, FALSE, FALSE, 10);
    gtk_box_pack_start (GTK_BOX (vbox), scrollwindow, TRUE, TRUE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), buttonvbox, FALSE, FALSE, 5);
	gtk_box_pack_start (GTK_BOX (process_page), vbox, TRUE, TRUE, 5);
     
    model = new_model ();													//vytvoreni noveho modelu

    treeview = gtk_tree_view_new_with_model (model);						//vytvoreni treeview
    gtk_tree_view_set_search_column (GTK_TREE_VIEW (treeview),				//definovani, podle jakeho slupce se bude vyhledavat
                                       COLUMN_NAME);

	gtk_tree_view_set_search_entry(GTK_TREE_VIEW (treeview), GTK_ENTRY(searchent));	//napojeni vzhledavaciho okna na treeview
    g_object_unref (model);

	add_columns (GTK_TREE_VIEW (treeview));									//pridani sloupcu do treeview
		
	g_signal_connect (button, "clicked",									//zpetne volani pro tlacitko, ktere ukonci vybrany proces
                        G_CALLBACK (quit_proces), treeview);
 
     gtk_container_add (GTK_CONTAINER (scrollwindow), treeview);			//pridani treeview do rolovaciho okna

	 return process_page;
}

static GtkWidget *createInfoPage()
{
	GtkWidget *info_page, *vbox, *vFrameSyBox, *vFrameHwBox, *hboxSy, *hboxHw;
	GtkWidget *systemFrame, *hardwareFrame;
	char *labeltext;

	info_page = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	vFrameSyBox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	vFrameHwBox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
	hboxSy = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
	hboxHw = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);

	systemFrame = gtk_frame_new("System");
	hardwareFrame = gtk_frame_new("Hardware");

	gtk_box_pack_start (GTK_BOX (info_page), vbox, TRUE, TRUE, 15);
	gtk_box_pack_start (GTK_BOX (vbox), systemFrame, FALSE, FALSE, 15);
	gtk_box_pack_start (GTK_BOX (vbox), hardwareFrame, FALSE, FALSE, 15);
	gtk_box_pack_start (GTK_BOX (hboxSy), vFrameSyBox, FALSE, FALSE, 30);
	gtk_box_pack_start (GTK_BOX (hboxHw), vFrameHwBox, FALSE, FALSE, 30);
	gtk_container_add (GTK_CONTAINER (systemFrame), hboxSy);
	gtk_container_add (GTK_CONTAINER (hardwareFrame), hboxHw);

	gtk_box_pack_start (GTK_BOX (vFrameSyBox), addLabelRow("Operating system", swhw_ptr->os, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameSyBox), addLabelRow("Kernel version", swhw_ptr->kernelRelease, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameSyBox), addLabelRow("Distribution", swhw_ptr->distro, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameSyBox), addLabelRow("Distro version", swhw_ptr->v_distro, 20), FALSE, FALSE, 7);
	
	labeltext = g_strdup_printf("%.0f MB", swhw_ptr->ram.totalRam);
	gtk_box_pack_start (GTK_BOX (vFrameHwBox), addLabelRow("Processor", swhw_ptr->cpu.nazev, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameHwBox), addLabelRow("Operation memory", labeltext, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameHwBox), addLabelRow("Graphics card", swhw_ptr->g_adapter, 20), FALSE, FALSE, 7);
	gtk_box_pack_start (GTK_BOX (vFrameHwBox), addLabelRow("Motherboard", swhw_ptr->board, 20), FALSE, FALSE, 7);
	g_free(labeltext);	

	return info_page;
}

GtkWidget * create_window()
{

	GtkWidget *menu_bar, *menu_item, *file_menu, *help_menu, *vbox, *notebook;
	GtkWidget *infoLabel, *memoryLabel, *cpuLabel, *processLabel;
	
	app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (app), WIDTH, HEIGHT);
    gtk_window_set_title (GTK_WINDOW (app), "System status check");


	infoLabel=gtk_label_new("General Info."); 
    memoryLabel=gtk_label_new(" Memory ");
    cpuLabel=gtk_label_new("    CPU    ");
    processLabel=gtk_label_new("Processes"); 
   

	menu_bar = gtk_menu_bar_new();
	file_menu = gtk_menu_new();
	help_menu = gtk_menu_new();

	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL,0);
	gtk_box_set_spacing(GTK_BOX(vbox), 5);

    notebook=gtk_notebook_new();
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);


	menu_item = gtk_menu_item_new_with_label("File");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), file_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);

	menu_item = gtk_menu_item_new_with_label("About");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), help_menu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);

	menu_item = gtk_menu_item_new_with_label("Print");
	gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), menu_item);
	g_signal_connect(menu_item, "activate", G_CALLBACK(start_printing),NULL);

	menu_item = gtk_menu_item_new_with_label("Quit");
	gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), menu_item);
	g_signal_connect(menu_item, "activate", G_CALLBACK(close_app),NULL);

	menu_item = gtk_menu_item_new_with_label("About");
	gtk_menu_shell_append(GTK_MENU_SHELL(help_menu), menu_item);
	g_signal_connect(menu_item, "activate", G_CALLBACK(about_app),NULL);
	
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createInfoPage(),infoLabel);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createMemoryPage(),  memoryLabel);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createCpuPage(),  cpuLabel);
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), createProcessPage(),  processLabel);
	

	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);   
	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0); 

    g_signal_connect (G_OBJECT (app), "destroy", G_CALLBACK(close_app), NULL);
    gtk_container_add (GTK_CONTAINER (app), vbox);

	return app;
	
}
